#include <Rcpp.h>
#include <RcppEigen.h>
#include "0_SeqStrucChange_aux.h"
#include "fluct_Gauss_margin.h"
#include "fluct_Gauss_mv.h"
#include "suplr_Gauss_margin.h"
#include "suplr_Gauss_mv.h"
#include "suplr_Gauss_copula.h"
#include "Gauss_aux.h"
#include "suplr_t_margin.h"
#include "suplr_t_copula.h"
#include "t_aux.h"

// [[Rcpp::depends(RcppEigen)]]
using namespace Rcpp;
using namespace Eigen;

using Eigen::MatrixXd;
using Eigen::VectorXd;
using Eigen::Map;

// from fluct_Gauss_marginal
// [[Rcpp::export]]
List fluct_GaussMarginal(NumericVector vecX){
  const Eigen::Map<VectorXd> importX(as<Map<VectorXd> >(vecX));
  const List res = fluct_GaussMarginal(importX);
  return res;
}

// [[Rcpp::export]]
List fluct_Squares(NumericVector vecX, double Omega = 1){
  const Eigen::Map<VectorXd> importX(as<Map<VectorXd> >(vecX));
  const List res = fluct_Squares(importX, Omega);
  return res;
}

// from fluct_Gauss_mv
// [[Rcpp::export]]
List fluct_GaussJoint_asym(NumericMatrix matX){
  const Map<MatrixXd> importX(as<Map<MatrixXd> >(matX));
  const List res = fluct_GaussJoint_asym(importX);
  return res;
}

// [[Rcpp::export]]
List fluct_GaussJoint_wildboot(NumericMatrix matX, const int B=200){
  const Map<MatrixXd> importX(as<Map<MatrixXd> >(matX));
  const List res = fluct_GaussJoint_wildboot(importX,B);
  return res;
}

// from suplr_Gauss_margin
// [[Rcpp::export]]
List suplr_GaussMarginal(NumericVector vecX){
  const Map<VectorXd> importX(as<Map<VectorXd> >(vecX));
  const List res = suplr_GaussMarginal(importX);
  return res;
}

// [[Rcpp::export]]
List suplr_Squares(NumericVector vecX, double Omega = 1){
  const Map<VectorXd> importX(as<Map<VectorXd> >(vecX));
  const List res = suplr_Squares(importX, Omega);
  return res;
}

// from suplr_Gauss_mv
// [[Rcpp::export]]
List suplr_GaussJoint_asym(NumericMatrix matX){
  const Map<MatrixXd> importX(as<Map<MatrixXd> >(matX));
  const List res = suplr_GaussJoint_asym(importX);
  return res;
}

// [[Rcpp::export]]
List suplr_GaussJoint_resboot(NumericMatrix matX, const int B=200){
  const Map<MatrixXd> importX(as<Map<MatrixXd> >(matX));
  const List res = suplr_GaussJoint_resboot(importX, B);
  return res;
}

// [[Rcpp::export]]
List suplr_GaussMarginal_GaussJoint_asym(NumericMatrix matX){
  const Map<MatrixXd> importX(as<Map<MatrixXd> >(matX));
  const List res = suplr_GaussMarginal_GaussJoint_asym(importX);
  return res;
}

// [[Rcpp::export]]
List suplr_GaussMarginal_GaussJoint_resboot(NumericMatrix matX, const int B=200){
  const Map<MatrixXd> importX(as<Map<MatrixXd> >(matX));
  const List res = suplr_GaussMarginal_GaussJoint_resboot(importX, B);
  return res;
}

// from suplr_Gauss_copula
// [[Rcpp::export]]
List suplr_GaussCopula_asym(NumericMatrix matX, const int niter=100){
  const Map<MatrixXd> importX(as<Map<MatrixXd> >(matX));
  const List res = suplr_GaussCopula_asym(importX);
  return res;
}

// [[Rcpp::export]]
List suplr_GaussCopula_resboot(NumericMatrix matX, const int niter=100, const int B=200){
  const Map<MatrixXd> importX(as<Map<MatrixXd> >(matX));
  const List res = suplr_GaussCopula_resboot(importX, B);
  return res;
}

// [[Rcpp::export]]
List suplr_tMarginal_GaussCopula_asym(NumericMatrix matX, const int niter=100){
  const Map<MatrixXd> importX(as<Map<MatrixXd> >(matX));
  const List res = suplr_tMarginal_GaussCopula_asym(importX, niter);
  return res;
}

// [[Rcpp::export]]
List suplr_tMarginal_GaussCopula_resboot(NumericMatrix matX, const int niter=100, const int B=200){
  const Map<MatrixXd> importX(as<Map<MatrixXd> >(matX));
  const List res = suplr_tMarginal_GaussCopula_resboot(importX, niter ,B);
  return res;
}

// from suplr_t_margin
// [[Rcpp::export]]
List suplr_tMarginal(NumericVector vecX, double nu=0, bool nu_const = true, int niter=100){
  const Map<VectorXd> importX(as<Map<VectorXd> >(vecX));
  const List res = suplr_tMarginal(importX, nu, nu_const, niter);
  return res;
}

// from suplr_t_copula
// [[Rcpp::export]]
List suplr_tMarginal_tCopula_asym(NumericMatrix matX, const int niter=100){
  const Map<MatrixXd> importX(as<Map<MatrixXd> >(matX));
  const List res = suplr_tMarginal_tCopula_asym(importX, niter);
  return res;
}

// [[Rcpp::export]]
List suplr_tMarginal_tCopula_resboot(NumericMatrix matX, const int niter=100, const int B=200){
  const Map<MatrixXd> importX(as<Map<MatrixXd> >(matX));
  const List res = suplr_tMarginal_tCopula_resboot(importX, niter,B);
  return res;
}

// [[Rcpp::export]]
List suplr_tCopula_asym(NumericMatrix matX){
  const Map<MatrixXd> importX(as<Map<MatrixXd> >(matX));
  const List res = suplr_tCopula_asym(importX);
  return res;
}

// [[Rcpp::export]]
List suplr_tCopula_resboot(NumericMatrix matX, const int B=200){
  const Map<MatrixXd> importX(as<Map<MatrixXd> >(matX));
  const List res = suplr_tCopula_resboot(importX,B);
  return res;
}

// functions from auxiliary files
// [[Rcpp::export]]
List mle_t(NumericMatrix matZ, double nu, int niter=1){
  const Map<MatrixXd> importZ(as<Map<MatrixXd> >(matZ));
  const List res = mle_t(importZ,nu,niter);
  return res;
}

// [[Rcpp::export]]
NumericVector mle_Gaussian_scores(NumericVector vecX, double sigma){
  const Map<VectorXd> importX(as<Map<VectorXd> >(vecX));
  const VectorXd vecScores = mle_Gaussian_scores(importX,sigma);
  return wrap(vecScores);
}
