#ifndef __GAUSS_AUX_INCLUDED__
#define __GAUSS_AUX_INCLUDED__
#include <Rcpp.h>
#include <RcppEigen.h>
#include <cmath>
#include "0_SeqStrucChange_aux.h"

// [[Rcpp::depends(RcppEigen)]]
using namespace Rcpp;
using namespace Eigen;

using Eigen::MatrixXd;
using Eigen::VectorXd;
using Eigen::Map;

// scores of centered univariate Gaussian distribution
VectorXd mle_Gaussian_scores(VectorXd vecX, double sigma){
  int n = vecX.rows();
  VectorXd vecScore(n);
  for(int t=0; t<n; t++){
    vecScore(t)= -0.5/sigma + pow(vecX(t),2)/(2*pow(sigma,2));
  }
  return vecScore;
}

// log-likelihood for i.i.d. sample from Gauss Copula
double logl_GaussCopula(MatrixXd matZ, MatrixXd matRho){
  const int n = matZ.rows();
  const int m = matZ.cols();
  double ll = -n*0.5*log(matRho.determinant());
  VectorXd vecObs_trans(m); 
  double delta; // Mahalanobis distance
  MatrixXd matRho_inv = matRho.inverse();
  MatrixXd matRho_trans(m,m);
  for (int i=0; i<m; i++){ 
    for(int j=0; j<m; j++){
    if(i==j){ matRho_trans(i,j)=matRho_inv(i,j)-1;}
    else{matRho_trans(i,j)=matRho_inv(i,j);}
  } }
  for(int t=0; t<n; t++){
    for(int j=0; j<m; j++){vecObs_trans(j) = R::qnorm(matZ(t,j),0,1,true,false);}
    delta = vecObs_trans.transpose() * matRho_trans * vecObs_trans; 
    ll = ll - 0.5 * delta;
  }
  return ll;
}

#endif
