#ifndef __FLUCT_GAUSS_IID_INCLUDED__
#define __FLUCT_GAUSS_IID_INCLUDED__
#include <Rcpp.h>
#include <RcppEigen.h>
#include <cmath>
#include "0_SeqStrucChange_aux.h"

// [[Rcpp::depends(RcppEigen)]]
using namespace Rcpp;
using namespace Eigen;

using Eigen::MatrixXd;
using Eigen::VectorXd;
using Eigen::Map;

// fluctuation test for constant marginal distribution against single break
List fluct_GaussMarginal(VectorXd vecX){
  const int n = vecX.size();
  double mu1_fs = vecX.sum()/n;
  double mu2_fs = ((vecX.array()).pow(2)).sum()/n;
  double mu3_fs = ((vecX.array()).pow(3)).sum()/n;
  double mu4_fs = ((vecX.array()).pow(4)).sum()/n;
  // inverse covariance matrix using full sample
  MatrixXd Omega(2,2);
  double det=(mu4_fs - pow(mu2_fs,2))*(mu2_fs - pow(mu1_fs,2))-pow((mu3_fs-mu2_fs*mu1_fs),2);
  Omega(0,0) = (mu4_fs - pow(mu2_fs,2))/det;
  Omega(1,0) = (mu1_fs*mu2_fs-mu3_fs)/det;
  Omega(0,1) = Omega(1,0);
  Omega(1,1) = (mu2_fs - pow(mu1_fs,2))/det;
  NumericVector vecQ(n);
  for(int j=2; j<(n-1); j++){
    VectorXd vecXsub(j);
    for(int t=0; t<j; t++){vecXsub(t)=vecX(t);}
    double mu1_ps = vecXsub.sum()/j;
    double mu2_ps = ((vecXsub.array()).pow(2)).sum()/j;
    VectorXd vecS(2);
    vecS(0)=mu1_ps - mu1_fs;
    vecS(1)=mu2_ps - mu2_fs;
    MatrixXd matQuadForm = vecS.transpose() * Omega * vecS;
    double qf = matQuadForm(0,0);
    vecQ[j] = j * pow(n,-0.5) * pow(qf,0.5);
  }
  double Qmax = max(vecQ);
  int lambda = which_max(vecQ)+1;
  // partial-sample ML-estimator
  VectorXd vecXsub1(lambda);
  for(int t=0; t<lambda; t++){vecXsub1(t)=vecX(t);}
  double mu1_ps1 = vecXsub1.sum()/lambda;
  double mu2_ps1 = ((vecXsub1.array()).pow(2)).sum()/lambda;
  VectorXd vecXsub2(n-lambda);
  for(int t=lambda; t<n; t++){vecXsub2(t-lambda)=vecX(t);}
  double mu1_ps2 = vecXsub2.sum()/(n-lambda);
  double mu2_ps2 = ((vecXsub2.array()).pow(2)).sum()/(n-lambda);
  double sigma0 = mu2_fs-pow(mu1_fs,2);
  double sigma1 = mu2_ps1-pow(mu1_ps1,2);
  double sigma2 = mu2_ps2-pow(mu1_ps2,2);
  return List::create(
    Named("Qmax") = Qmax,
    Named("lambda") = lambda,
    Named("mu0") = mu1_fs,
    Named("mu1") = mu1_ps1,
    Named("mu2") = mu1_ps2,
    Named("sigma0") = sigma0,
    Named("sigma1") = sigma1,
    Named("sigma2") = sigma2);
}

// fluctuation test for constant marginal distribution with zero mean against single break
List fluct_Squares(VectorXd vecX, double Omega = 1){
  const int n = vecX.size();
  double mu2_fs = ((vecX.array()).pow(2)).sum()/n;
  double mu4_fs = ((vecX.array()).pow(4)).sum()/n;
  // inverse covariance matrix using full sample
  NumericVector vecQ(n);
  for(int j=2; j<(n-1); j++){
    VectorXd vecXsub(j);
    for(int t=0; t<j; t++){vecXsub(t)=vecX(t);}
    double mu2_ps = ((vecXsub.array()).pow(2)).sum()/j;
    double S=mu2_ps - mu2_fs;
    vecQ[j] = j * pow(n,-0.5) * pow(pow(S,2)/Omega,0.5);
  }
  double Qmax = max(vecQ);
  int lambda = which_max(vecQ)+1;
  // partial-sample ML-estimator
  VectorXd vecXsub1(lambda);
  for(int t=0; t<lambda; t++){vecXsub1(t)=vecX(t);}
  double sigma1 = ((vecXsub1.array()).pow(2)).sum()/lambda;
  VectorXd vecXsub2(n-lambda);
  for(int t=lambda; t<n; t++){vecXsub2(t-lambda)=vecX(t);}
  double sigma2 = ((vecXsub2.array()).pow(2)).sum()/(n-lambda);
  return List::create(
    Named("Qmax") = Qmax,
    Named("lambda") = lambda,
    Named("sigma0") = mu2_fs,
    Named("sigma1") = sigma1,
    Named("sigma2") = sigma2);
}

#endif
