#ifndef __FLUCT_GAUSS_MV_INCLUDED__
#define __FLUCT_GAUSS_MV_INCLUDED__
#include <Rcpp.h>
#include <RcppEigen.h>
#include <cmath>
#include "0_SeqStrucChange_aux.h"
#include "fluct_Gauss_margin.h"

// [[Rcpp::depends(RcppEigen)]]
using namespace Rcpp;
using namespace Eigen;

using Eigen::MatrixXd;
using Eigen::VectorXd;
using Eigen::Map;

// fluctuation test for constant correlation against single break
// margins are tested for constant mean/variance against single break in every dimension
List fluct_GaussJoint_asym(MatrixXd matX){
  const int n = matX.rows();
  const int m = matX.cols();
  const int l = 0.5*m*(m-1);
  int lambda_marg[m];
  double Qmax_marg[m];
  // standardizing data
  MatrixXd matZ(n,m);
  for(int i = 0; i<m; i++){
    VectorXd vecX = matX.col(i);
    List param = fluct_GaussMarginal(vecX);
    Qmax_marg[i] = param["Qmax"];
    lambda_marg[i] = param["lambda"];
    double mu1 = param["mu1"];
    double mu2 = param["mu2"];
    double mu0 = param["mu0"];
    double sigma1 = pow(param["sigma1"],0.5);
    double sigma2 = pow(param["sigma2"],0.5);
    double sigma0 = pow(param["sigma0"],0.5);
    if(Qmax_marg[i] > 1.584){
      for(int t=0; t<lambda_marg[i]; t++){ matZ(t,i)=(matX(t,i)-mu1)/sigma1; }
      for(int t=lambda_marg[i]; t<n; t++){ matZ(t,i)=(matX(t,i)-mu2)/sigma2; }
    }
    else{ 
      for(int t=0; t<n; t++){ matZ(t,i)=(matX(t,i)-mu0)/sigma0; }
    }
  }
  // full-sample estimation
  VectorXd vecM_fs = Mij_coef(matZ);
  MatrixXd matO = Oij_boot(matZ,200);
  MatrixXd matO_inv = matO.inverse();
  VectorXd vecM_ps(l);
  VectorXd vecM_diff(l);
  // target vector
  NumericVector vecQ(n);
  for(int j=2; j<(n-1); j++){
    MatrixXd matZsub(j,m);
    for(int i=0; i<m; i++){ for(int t=0; t<j; t++){ matZsub(t,i)=matZ(t,i);}}
    vecM_ps = Mij_coef(matZsub);
    vecM_diff = vecM_fs - vecM_ps;
    MatrixXd matQuadForm = vecM_diff.transpose() * matO_inv * vecM_diff;
    double qf = matQuadForm(0,0);
    vecQ[j] = j * pow(n,-1) * pow(qf,0.5);
    }
  double Qmax = max(vecQ);
  int lambda = which_max(vecQ)+1;
  // partial-sample ML-estimator
  MatrixXd matZsub1(lambda,m);
  MatrixXd matZsub2(n-lambda,m);
  for(int i=0; i<m; i++){ 
    for(int t=0; t<lambda; t++){ matZsub1(t,i)=matZ(t,i);}
    for(int t=lambda; t<n; t++){ matZsub2(t-lambda,i)=matZ(t,i);}
    }
  MatrixXd rho_fs = corr_mat(matZ);
  MatrixXd rho_ps1 = corr_mat(matZsub1);
  MatrixXd rho_ps2 = corr_mat(matZsub2);
  // convert for output
  NumericVector vecQmax_marg(m);
  IntegerVector vecLambda_marg(m);
  for(int i=0; i<m; i++){
    vecQmax_marg[i]=Qmax_marg[i];
    vecLambda_marg[i]=lambda_marg[i];
    }
  return List::create(
    Named("Qmax") = Qmax,
    Named("lambda") = lambda,
    Named("matZ")=matZ,
    Named("Qmax_margins") = vecQmax_marg,
    Named("lambda_margins") = vecLambda_marg,
    Named("rho0") = wrap(rho_fs),
    Named("rho1") = wrap(rho_ps1),
    Named("rho2") = wrap(rho_ps2));
}

// fluctuation test for constant correlation using wild bootstrap from Demetrescu and Wied (2018)
// margins are tested for constant mean/variance against single break in every dimension
List fluct_GaussJoint_wildboot(MatrixXd matX, int B){
  int n = matX.rows();
  int m = matX.cols();
  IntegerVector vecInd(n);
  for(int t=0; t<n; t++){vecInd[t]=t;}
  List res = fluct_GaussJoint_asym(matX);
  // target arrays for marginal parameters
  double Qmax_marg[m];
  double mu0[m];
  double mu1[m];
  double mu2[m];
  double sigma0[m];
  double sigma1[m];
  double sigma2[m];
  int lambda_marg[m];
  // determining marginal parameters
  for(int i=0; i<m; i++){
    VectorXd vecX = matX.col(i);
    List param = fluct_GaussMarginal(vecX);
    Qmax_marg[i] = param["Qmax"];
    lambda_marg[i] = param["lambda"];
    mu0[i] = param["mu0"];
    mu1[i] = param["mu1"];
    mu2[i] = param["mu2"];
    sigma0[i] = param["sigma0"];
    sigma1[i] = param["sigma1"];
    sigma2[i] = param["sigma2"];
  }
  // target vector for test statistics
  NumericVector vecQboot(B);
  // wild bootstrap sample
  for(int b=0; b<B; b++){
    MatrixXd matXboot(n,m);
    IntegerVector vecInd_boot = boot(vecInd,n);
    for(int i=0; i<m; i++){
      VectorXd vecX(n);
      for(int t=0; t<n; t++){ vecX(t) = matX(vecInd_boot[t],i); }
      if(Qmax_marg[i]>1.584){
        VectorXd vecXsub1(lambda_marg[i]);
        for(int t=0; t<lambda_marg[i]; t++){vecXsub1(t)=vecX(t);}
        VectorXd vecXsub2(n-lambda_marg[i]);
        for(int t=lambda_marg[i]; t<n; t++){vecXsub2(t-lambda_marg[i])=vecX(t);}
        double mu1_boot = vecXsub1.sum()/lambda_marg[i];
        double mu2_boot = vecXsub2.sum()/(n-lambda_marg[i]);
        double sigma1_boot = ((vecXsub1.array()).pow(2)).sum()/lambda_marg[i] - pow(mu1_boot,2);
        double sigma2_boot = ((vecXsub2.array()).pow(2)).sum()/(n-lambda_marg[i]) - pow(mu2_boot,2);
        for(int t=0; t<lambda_marg[i]; t++){
          matXboot(t,i)=mu1[i]+(vecXsub1[t]-mu1_boot)*pow(sigma1[i],0.5)/pow(sigma1_boot,0.5);}
        for(int t=lambda_marg[i]; t<n; t++){
          matXboot(t,i)=mu2[i]+(vecXsub2[t-lambda_marg[i]]-mu2_boot)*pow(sigma2[i],0.5)/pow(sigma2_boot,0.5);}
      }
      else{
        double mu0_boot = vecX.sum()/n;
        double sigma0_boot = ((vecX.array()).pow(2)).sum()/n - pow(mu0_boot,2);
        for(int t=0; t<n; t++){
          matXboot(t,i)=mu0[i]+(vecX[t]-mu0_boot)*pow(sigma0[i],0.5)/pow(sigma0_boot,0.5);}
      }
    }
    List res_boot = fluct_GaussJoint_asym(matXboot);
    double Qboot = res_boot["Qmax"];
    vecQboot[b] = Qboot;
  }
  // compute p-value
  double Qmax = res["Qmax"];
  double pval = mean(vecQboot > Qmax);
  // convert for output
  NumericVector vecQmax_marg(m);
  IntegerVector vecLambda_marg(m);
  for(int i=0; i<m; i++){
    vecQmax_marg[i]=Qmax_marg[i];
    vecLambda_marg[i]=lambda_marg[i];}
  return Rcpp::List::create(
    Named("pval_boot") = pval,
    Named("Qmax") = res["Qmax"],
    Named("lambda") = res["lambda"],
    Named("matZ")=res["matZ"],
    Named("Qmax_margins")=vecQmax_marg,
    Named("lambda_margins")=vecLambda_marg,
    Named("rho0") = res["rho0"],
    Named("rho1") = res["rho1"],
    Named("rho2") = res["rho2"]);
}

#endif
