#ifndef __SUPLR_GAUSS_COPULA_INCLUDED__
#define __SUPLR_GAUSS_COPULA_INCLUDED__
#include <Rcpp.h>
#include <RcppEigen.h>
#include <cmath>
#include "0_SeqStrucChange_aux.h"
#include "suplr_Gauss_margin.h"
#include "Gauss_aux.h"
#include "suplr_t_margin.h"
#include "t_aux.h"

// [[Rcpp::depends(RcppEigen)]]
using namespace Rcpp;
using namespace Eigen;

using Eigen::Map;
using Eigen::MatrixXd;
using Eigen::VectorXd;
using Eigen::Lower;

// suplr-test for constant Gaussian Copula against single break
// matZ is assumed to be i.i.d. normal (GARCH-residuals)
List suplr_GaussCopula_asym(MatrixXd matZ){
  const int m = matZ.cols();
  const int n = matZ.rows();
  MatrixXd matU(n,m);
  for(int i = 0; i<m; i++){
    for(int t=0; t<n; t++){
      matU(t,i)= R::pnorm(matZ(t,i),0,1,true,false); 
    }
  }
  MatrixXd rho0 = corr_mat(matU);
  double ll0 = logl_GaussCopula(matU,rho0);
  MatrixXd rho1(m,m);
  MatrixXd rho2(m,m);
  // initialize target vector
  NumericVector vecLR(n);
  // calculate lr-statistic for set of change-point candidates
  for (int j = 0.15*n; j < 0.85*n+1; j++) {
    MatrixXd matZsub1(j,m);
    MatrixXd matZsub2(n-j,m);
    for(int i=0; i<m; i++){
      for(int t=0; t<j; t++){
        matZsub1(t,i)=matU(t,i);}
      for(int t=j; t<n; t++){
        matZsub2(t-j,i)=matU(t,i);}
    }
    rho1 = corr_mat(matZsub1);
    rho2 = corr_mat(matZsub2);
    vecLR[j]= 2*(logl_GaussCopula(matZsub1,rho1)+logl_GaussCopula(matZsub2,rho2)-ll0);
  }
  double suplr = max(vecLR);
  int lambda = which_max(vecLR)+1;
  // partial sample ML-estimator
  MatrixXd matZ1(lambda,m);
  MatrixXd matZ2(n-lambda,m);
  for(int i=0; i<m; i++){
    for(int t=0; t<lambda; t++){
      matZ1(t,i)=matU(t,i);}
    for(int t=lambda; t<n; t++){
      matZ2(t-lambda,i)=matU(t,i);}
  }
  rho1 = corr_mat(matZ1);
  rho2 = corr_mat(matZ2);
  // return list with test result
  return List::create(
    Named("suplr")=suplr,
    Named("lambda")=lambda+1,
    Named("matU")=matU,
    Named("rho0")=rho0,
    Named("rho1")=rho1,
    Named("rho2")=rho2);
}

// suplr-test for constant Gaussian Copula against single break using residual bootstrap
// matZ is assumed to be i.i.d. normal (GARCH-residuals)
List suplr_GaussCopula_resboot(MatrixXd matZ, const int B = 200){
  const int m = matZ.cols();
  const int n = matZ.rows();
  IntegerVector vecInd(n);
  for(int t=0; t<n; t++){vecInd[t]=t;}
  MatrixXd matU(n,m);
  for(int i = 0; i<m; i++){
    for(int t=0; t<n; t++){
      matU(t,i)= R::pnorm(matZ(t,i),0,1,true,false); 
    }
  }
  MatrixXd rho0 = corr_mat(matU);
  double ll0 = logl_GaussCopula(matU,rho0);
  MatrixXd rho1(m,m);
  MatrixXd rho2(m,m);
  // initialize target vector
  NumericVector vecLR(n);
  // calculate lr-statistic for set of change-point candidates
  for (int j = 0.15*n; j < 0.85*n+1; j++) {
    MatrixXd matUsub1(j,m);
    MatrixXd matUsub2(n-j,m);
    for(int i=0; i<m; i++){
      for(int t=0; t<j; t++){
        matUsub1(t,i)=matU(t,i);}
      for(int t=j; t<n; t++){
        matUsub2(t-j,i)=matU(t,i);}
    }
    rho1 = corr_mat(matUsub1);
    rho2 = corr_mat(matUsub2);
    vecLR[j]= 2*(logl_GaussCopula(matUsub1,rho1)+logl_GaussCopula(matUsub2,rho2)-ll0);
  }
  double suplr = max(vecLR);
  int lambda = which_max(vecLR)+1;
  // partial sample ML-estimator
  MatrixXd matZ1(lambda,m);
  MatrixXd matZ2(n-lambda,m);
  for(int i=0; i<m; i++){
    for(int t=0; t<lambda; t++){
      matZ1(t,i)=matU(t,i);}
    for(int t=lambda; t<n; t++){
      matZ2(t-lambda,i)=matU(t,i);}
  }
  rho1 = corr_mat(matZ1);
  rho2 = corr_mat(matZ2);
  // residual bootstrap
  NumericVector vecSUPLRboot(B);
  MatrixXd matUboot(n,m);
  IntegerVector vecInd_boot(n);
  for(int b=0; b<B; b++){
    vecInd_boot = boot(vecInd,n);
    for(int i=0; i<m; i++){ for(int t=0; t<n; t++){ matUboot(t,i)=matU(vecInd_boot(t),i); }}
    rho0 = corr_mat(matUboot);
    ll0 = logl_GaussCopula(matUboot,rho0);
    MatrixXd rho1(m,m);
    MatrixXd rho2(m,m);
    // initialize target vector
    NumericVector vecLR(n);
    // calculate lr-statistic for set of change-point candidates
    for (int j = 0.15*n; j < 0.85*n+1; j++) {
      MatrixXd matUsub1(j,m);
      MatrixXd matUsub2(n-j,m);
      for(int i=0; i<m; i++){
        for(int t=0; t<j; t++){
          matUsub1(t,i)=matUboot(t,i);}
        for(int t=j; t<n; t++){
          matUsub2(t-j,i)=matUboot(t,i);}
      }
      rho1 = corr_mat(matUsub1);
      rho2 = corr_mat(matUsub2);
      vecLR[j]= 2*(logl_GaussCopula(matUsub1,rho1)+logl_GaussCopula(matUsub2,rho2)-ll0);
    }
    vecSUPLRboot(b)=max(vecLR);
  }
  // p-value from bootstrap
  double pval_boot = mean(vecSUPLRboot > suplr);
  // return list with test result
  return List::create(
    Named("pval_boot") = pval_boot,
    Named("suplr")=suplr,
    Named("lambda")=lambda+1,
    Named("matU")=matU,
    Named("rho0")=rho0,
    Named("rho1")=rho1,
    Named("rho2")=rho2);
}

// two-step suplr-test for constant correlation matrix against single break
// margins are assumed to follow t-distributions
// matX is tested for constant mean/variance in each dimension against single break
List suplr_tMarginal_GaussCopula_asym(MatrixXd matX, const int niter = 100){
  const int m = matX.cols();
  const int n = matX.rows();
  MatrixXd matU(n,m);
  // testing and estimating at the margins
  int lambda_marg[m];
  double suplr_marg[m];
  for(int i = 0; i<m; i++){
    VectorXd vecX = matX.col(i);
    List param_i = suplr_tMarginal(vecX,0,true,niter);
    suplr_marg[i] = param_i["suplr"];
    lambda_marg[i] = param_i["lambda"];
    if(suplr_marg[i] > 11.72){
      double nu1 = param_i["nu1"];
      double nu2 = param_i["nu2"];
      double mu1 = param_i["mu1"];
      double mu2 = param_i["mu2"];
      double xi1_sqrt = pow(param_i["xi1"],0.5);
      double xi2_sqrt = pow(param_i["xi2"],0.5);
      for(int t=0; t<lambda_marg[i]; t++){ 
        double x_norm = (matX(t,i)-mu1)/xi1_sqrt;
        matU(t,i)= R::pt(x_norm,nu1,true,false); 
      }
      for(int t=lambda_marg[i]; t<n; t++){
        double x_norm = (matX(t,i)-mu2)/xi2_sqrt;
        matU(t,i)= R::pt(x_norm,nu2,true,false); 
      }
    }
    else{
      double mu0 = param_i["mu0"];
      double nu0 = param_i["nu0"];
      double xi0_sqrt = pow(param_i["xi0"],0.5);
      for(int t=0; t<n; t++){
        double x_norm = (matX(t,i)-mu0)/xi0_sqrt;
        matU(t,i)= R::pt(x_norm,nu0,true,false); 
      }
    }
  }
  MatrixXd rho0 = corr_mat(matU);
  double ll0 = logl_GaussCopula(matU,rho0);
  MatrixXd rho1(m,m);
  MatrixXd rho2(m,m);
  // initialize target vector
  NumericVector vecLR(n);
  // calculate lr-statistic for set of change-point candidates
  for (int j = 0.15*n; j < 0.85*n+1; j++) {
    MatrixXd matZsub1(j,m);
    MatrixXd matZsub2(n-j,m);
    for(int i=0; i<m; i++){
      for(int t=0; t<j; t++){
        matZsub1(t,i)=matU(t,i);}
      for(int t=j; t<n; t++){
        matZsub2(t-j,i)=matU(t,i);}
    }
    rho1 = corr_mat(matZsub1);
    rho2 = corr_mat(matZsub2);
    vecLR[j]= 2*(logl_GaussCopula(matZsub1,rho1)+logl_GaussCopula(matZsub2,rho2)-ll0);
  }
  double suplr = max(vecLR);
  int lambda = which_max(vecLR)+1;
  // partial sample ML-estimator
  MatrixXd matZ1(lambda,m);
  MatrixXd matZ2(n-lambda,m);
  for(int i=0; i<m; i++){
    for(int t=0; t<lambda; t++){
      matZ1(t,i)=matU(t,i);}
    for(int t=lambda; t<n; t++){
      matZ2(t-lambda,i)=matU(t,i);}
  }
  rho1 = corr_mat(matZ1);
  rho2 = corr_mat(matZ2);
  // convert for output
  NumericVector vecSuplr_marg(m);
  IntegerVector vecLambda_marg(m);
  for(int i=0; i<m; i++){
    vecSuplr_marg[i]=suplr_marg[i];
    vecLambda_marg[i]=lambda_marg[i];
  }
  // return list with test result
  return List::create(
    Named("suplr")=suplr,
    Named("lambda")=lambda+1,
    Named("matU")=matU,
    Named("suplr_margins")=vecSuplr_marg,
    Named("lambda_margins")=vecLambda_marg,
    Named("rho0")=rho0,
    Named("rho1")=rho1,
    Named("rho2")=rho2);
}

// two-step suplr-test for constant correlation matrix against single break using residual bootstrap
// margins are assumed to follow t-distributions
// matX is tested for constant mean/variance in each dimension against single break
List suplr_tMarginal_GaussCopula_resboot(MatrixXd matX, const int niter = 100, const int B = 200){
  const int m = matX.cols();
  const int n = matX.rows();
  IntegerVector vecInd(n);
  for(int t=0; t<n; t++){vecInd[t]=t;}
  MatrixXd matU(n,m);
  // testing and estimating at the margins
  int lambda_marg[m];
  double suplr_marg[m];
  for(int i = 0; i<m; i++){
    VectorXd vecX = matX.col(i);
    List param_i = suplr_tMarginal(vecX,0,true,niter);
    suplr_marg[i] = param_i["suplr"];
    lambda_marg[i] = param_i["lambda"];
    if(suplr_marg[i] > 11.72){
      double nu1 = param_i["nu1"];
      double nu2 = param_i["nu2"];
      double mu1 = param_i["mu1"];
      double mu2 = param_i["mu2"];
      double xi1_sqrt = pow(param_i["xi1"],0.5);
      double xi2_sqrt = pow(param_i["xi2"],0.5);
      for(int t=0; t<lambda_marg[i]; t++){ 
        double x_norm = (matX(t,i)-mu1)/xi1_sqrt;
        matU(t,i)= R::pt(x_norm,nu1,true,false); 
      }
      for(int t=lambda_marg[i]; t<n; t++){
        double x_norm = (matX(t,i)-mu2)/xi2_sqrt;
        matU(t,i)= R::pt(x_norm,nu2,true,false); 
      }
    }
    else{
      double mu0 = param_i["mu0"];
      double nu0 = param_i["nu0"];
      double xi0_sqrt = pow(param_i["xi0"],0.5);
      for(int t=0; t<n; t++){
        double x_norm = (matX(t,i)-mu0)/xi0_sqrt;
        matU(t,i)= R::pt(x_norm,nu0,true,false); 
      }
    }
  }
  MatrixXd rho0 = corr_mat(matU);
  double ll0 = logl_GaussCopula(matU,rho0);
  MatrixXd rho1(m,m);
  MatrixXd rho2(m,m);
  // initialize target vector
  NumericVector vecLR(n);
  // calculate lr-statistic for set of change-point candidates
  for (int j = 0.15*n; j < 0.85*n+1; j++) {
    MatrixXd matUsub1(j,m);
    MatrixXd matUsub2(n-j,m);
    for(int i=0; i<m; i++){
      for(int t=0; t<j; t++){
        matUsub1(t,i)=matU(t,i);}
      for(int t=j; t<n; t++){
        matUsub2(t-j,i)=matU(t,i);}
    }
    rho1 = corr_mat(matUsub1);
    rho2 = corr_mat(matUsub2);
    vecLR[j]= 2*(logl_GaussCopula(matUsub1,rho1)+logl_GaussCopula(matUsub2,rho2)-ll0);
  }
  double suplr = max(vecLR);
  int lambda = which_max(vecLR)+1;
  // partial sample ML-estimator
  MatrixXd matZ1(lambda,m);
  MatrixXd matZ2(n-lambda,m);
  for(int i=0; i<m; i++){
    for(int t=0; t<lambda; t++){
      matZ1(t,i)=matU(t,i);}
    for(int t=lambda; t<n; t++){
      matZ2(t-lambda,i)=matU(t,i);}
  }
  rho1 = corr_mat(matZ1);
  rho2 = corr_mat(matZ2);
  // residual bootstrap
  NumericVector vecSUPLRboot(B);
  MatrixXd matUboot(n,m);
  IntegerVector vecInd_boot(n);
  for(int b=0; b<B; b++){
    vecInd_boot = boot(vecInd,n);
    for(int i=0; i<m; i++){ for(int t=0; t<n; t++){ matUboot(t,i)=matU(vecInd_boot(t),i); }}
    rho0 = corr_mat(matUboot);
    ll0 = logl_GaussCopula(matUboot,rho0);
    MatrixXd rho1(m,m);
    MatrixXd rho2(m,m);
    // initialize target vector
    NumericVector vecLR(n);
    // calculate lr-statistic for set of change-point candidates
    for (int j = 0.15*n; j < 0.85*n+1; j++) {
      MatrixXd matUsub1(j,m);
      MatrixXd matUsub2(n-j,m);
      for(int i=0; i<m; i++){
        for(int t=0; t<j; t++){
          matUsub1(t,i)=matUboot(t,i);}
        for(int t=j; t<n; t++){
          matUsub2(t-j,i)=matUboot(t,i);}
      }
      rho1 = corr_mat(matUsub1);
      rho2 = corr_mat(matUsub2);
      vecLR[j]= 2*(logl_GaussCopula(matUsub1,rho1)+logl_GaussCopula(matUsub2,rho2)-ll0);
    }
    vecSUPLRboot(b)=max(vecLR);
  }
  // p-value from bootstrap
  double pval_boot = mean(vecSUPLRboot > suplr);
  // convert for output
  NumericVector vecSuplr_marg(m);
  IntegerVector vecLambda_marg(m);
  for(int i=0; i<m; i++){
    vecSuplr_marg[i]=suplr_marg[i];
    vecLambda_marg[i]=lambda_marg[i];
  }
  // return list with test result
  return List::create(
    Named("pval_boot") = pval_boot,
    Named("suplr")=suplr,
    Named("lambda")=lambda+1,
    Named("matU")=matU,
    Named("suplr_margins")=vecSuplr_marg,
    Named("lambda_margins")=vecLambda_marg,
    Named("rho0")=rho0,
    Named("rho1")=rho1,
    Named("rho2")=rho2);
}

#endif
