#ifndef __SUPLR_GAUSS_IID_INCLUDED__
#define __SUPLR_GAUSS_IID_INCLUDED__
#include <Rcpp.h>
#include <RcppEigen.h>
#include <cmath>
#include "0_SeqStrucChange_aux.h"
#include "Gauss_aux.h"

// [[Rcpp::depends(RcppEigen)]]
using namespace Rcpp;
using namespace Eigen;

using Eigen::MatrixXd;
using Eigen::VectorXd;
using Eigen::Map;

// lr-test for constant marginal distribution under normality against single break
List suplr_GaussMarginal(VectorXd vecX){
  const int n = vecX.size();
  // moments under the null
  const double mu0=vecX.sum()/n;
  const double sigma0=((vecX.array()).pow(2)).sum()/n - pow(mu0,2);
  // initialize target array
  NumericVector vecLR(n);
  double mu1;
  double mu2;
  double sigma1;
  double sigma2;
  // loop over potential breakpoints
  for (int j = 0.15*n; j < 0.85*n+1; j++) {
  VectorXd vecXsub1(j);
  for(int t=0; t<j; t++){vecXsub1(t)=vecX(t);}
  VectorXd vecXsub2(n-j);
  for(int t=j; t<n; t++){vecXsub2(t-j)=vecX(t);}
  mu1 = vecXsub1.sum()/j;
  mu2 = vecXsub2.sum()/(n-j);
  sigma1=((vecXsub1.array()).pow(2)).sum()/j - pow(mu1,2);
  sigma2=((vecXsub2.array()).pow(2)).sum()/(n-j) - pow(mu2,2);
  vecLR[j-1] = n*log(sigma0) - j*log(sigma1) - (n-j)*log(sigma2); 
  }
  double suplr = max(vecLR);
  int lambda = which_max(vecLR)+1;
  // partial-sample ML-estimator
  VectorXd vecXsub1(lambda);
  VectorXd vecXsub2(n-lambda);
  for(int t=0; t<lambda; t++){vecXsub1(t)=vecX(t);}
  for(int t=lambda; t<n; t++){vecXsub2(t-lambda)=vecX(t);}
  mu1 = vecXsub1.sum()/lambda;
  mu2 = vecXsub2.sum()/(n-lambda);
  sigma1=((vecXsub1.array()).pow(2)).sum()/lambda - pow(mu1,2);
  sigma2=((vecXsub2.array()).pow(2)).sum()/(n-lambda) - pow(mu2,2);
  // return list with test result
  return List::create(
      Named("suplr")=suplr,
      Named("lambda")=lambda,
      Named("mu0")=mu0,
      Named("mu1")=mu1,
      Named("mu2")=mu2,
      Named("sigma0")=sigma0,
      Named("sigma1")=sigma1,
      Named("sigma2")=sigma2);
}

// lr-test for constant marginal distribution under normality against single break
List suplr_Squares(VectorXd vecX_raw, double Omega = 1){
  const int n = vecX_raw.size();
  VectorXd vecX(n);
  double mu0 = vecX_raw.sum()/n;
  for(int t=0; t<n; t++){ vecX(t)=vecX_raw(t)-mu0;}
  // moments under the null
  const double sigma0=((vecX.array()).pow(2)).sum()/n;
  // initialize target array
  NumericVector vecLR(n);
  double sigma1;
  double sigma2;
  // loop over potential breakpoints
  for (int j = 0.15*n; j < 0.85*n+1; j++) {
    VectorXd vecXsub1(j);
    for(int t=0; t<j; t++){vecXsub1(t)=vecX(t);}
    VectorXd vecXsub2(n-j);
    for(int t=j; t<n; t++){vecXsub2(t-j)=vecX(t);}
    sigma1=((vecXsub1.array()).pow(2)).sum()/j;
    sigma2=((vecXsub2.array()).pow(2)).sum()/(n-j);
    vecLR[j-1] = (n*log(sigma0) - j*log(sigma1) - (n-j)*log(sigma2))/Omega; 
  }
  double suplr = max(vecLR);
  int lambda = which_max(vecLR)+1;
  // partial-sample ML-estimator
  VectorXd vecXsub1(lambda);
  VectorXd vecXsub2(n-lambda);
  for(int t=0; t<lambda; t++){vecXsub1(t)=vecX(t);}
  for(int t=lambda; t<n; t++){vecXsub2(t-lambda)=vecX(t);}
  sigma1=((vecXsub1.array()).pow(2)).sum()/lambda;
  sigma2=((vecXsub2.array()).pow(2)).sum()/(n-lambda);
  // return list with test result
  return List::create(
    Named("suplr")=suplr,
    Named("lambda")=lambda,
    Named("sigma0")=sigma0,
    Named("sigma1")=sigma1,
    Named("sigma2")=sigma2);
}

#endif
