#ifndef __SUPLR_GAUSS_MV_INCLUDED__
#define __SUPLR_GAUSS_MV_INCLUDED__
#include <Rcpp.h>
#include <RcppEigen.h>
#include <cmath>
#include "0_SeqStrucChange_aux.h"
#include "suplr_Gauss_margin.h"
#include "Gauss_aux.h"

// [[Rcpp::depends(RcppEigen)]]
using namespace Rcpp;
using namespace Eigen;

using Eigen::MatrixXd;
using Eigen::VectorXd;
using Eigen::Map;

// suplr-test for constant correlation matrix against single break
// matZ is assumed to be i.i.d. normal (GARCH-residuals)
List suplr_GaussJoint_asym(MatrixXd matZ){
  const int m = matZ.cols();
  const int n = matZ.rows();
  MatrixXd rho0 = corr_mat(matZ);
  MatrixXd rho1(m,m);
  MatrixXd rho2(m,m);
  // initialize target vector
  NumericVector vecLR(n);
  // calculate lr-statistic for set of change-point candidates
  for (int j = 0.15*n; j < 0.85*n+1; j++) {
    MatrixXd matZsub1(j,m);
    MatrixXd matZsub2(n-j,m);
    for(int i=0; i<m; i++){
      for(int t=0; t<j; t++){
        matZsub1(t,i)=matZ(t,i);}
      for(int t=j; t<n; t++){
        matZsub2(t-j,i)=matZ(t,i);}
    }
    rho1 = corr_mat(matZsub1);
    rho2 = corr_mat(matZsub2);
    vecLR[j]= n*log(rho0.determinant()) - j*log(rho1.determinant()) - (n-j)*log(rho2.determinant());
  }
  double suplr = max(vecLR);
  int lambda = which_max(vecLR)+1;
  // partial sample ML-estimator
  MatrixXd matZ1(lambda,m);
  MatrixXd matZ2(n-lambda,m);
  for(int i=0; i<m; i++){
    for(int t=0; t<lambda; t++){
      matZ1(t,i)=matZ(t,i);}
    for(int t=lambda; t<n; t++){
      matZ2(t-lambda,i)=matZ(t,i);}
  }
  rho1 = corr_mat(matZ1);
  rho2 = corr_mat(matZ2);
  // return list with test result
  return List::create(
    Named("suplr")=suplr,
    Named("lambda")=lambda+1,
    Named("rho0")=wrap(rho0),
    Named("rho1")=wrap(rho1),
    Named("rho2")=wrap(rho2));
}

// suplr-test for constant correlation matrix against single break using residual bootstrap
// matZ is assumed to be i.i.d. normal (GARCH-residuals)
List suplr_GaussJoint_resboot(MatrixXd matZ, const int B = 200){
  const int m = matZ.cols();
  const int n = matZ.rows();
  IntegerVector vecInd(n);
  for(int t=0; t<n; t++){vecInd[t]=t;}
  MatrixXd rho0 = corr_mat(matZ);
  MatrixXd rho1(m,m);
  MatrixXd rho2(m,m);
  // initialize target vector
  NumericVector vecLR(n);
  // calculate lr-statistic for set of change-point candidates
  for (int j = 0.15*n; j < 0.85*n+1; j++) {
    MatrixXd matZsub1(j,m);
    MatrixXd matZsub2(n-j,m);
    for(int i=0; i<m; i++){
      for(int t=0; t<j; t++){
        matZsub1(t,i)=matZ(t,i);}
      for(int t=j; t<n; t++){
        matZsub2(t-j,i)=matZ(t,i);}
    }
    rho1 = corr_mat(matZsub1);
    rho2 = corr_mat(matZsub2);
    vecLR[j]= n*log(rho0.determinant()) - j*log(rho1.determinant()) - (n-j)*log(rho2.determinant());
  }
  double suplr = max(vecLR);
  int lambda = which_max(vecLR)+1;
  // partial sample ML-estimator
  MatrixXd matZ1(lambda,m);
  MatrixXd matZ2(n-lambda,m);
  for(int i=0; i<m; i++){
    for(int t=0; t<lambda; t++){
      matZ1(t,i)=matZ(t,i);}
    for(int t=lambda; t<n; t++){
      matZ2(t-lambda,i)=matZ(t,i);}
  }
  rho1 = corr_mat(matZ1);
  rho2 = corr_mat(matZ2);
  // residual bootstrap
  NumericVector vecSUPLRboot(B);
  MatrixXd rho0_boot;
  MatrixXd rho1_boot;
  MatrixXd rho2_boot;
  MatrixXd matZboot(n,m);
  for(int b=0; b<B; b++){
    IntegerVector vecInd_boot = boot(vecInd,n);
    for(int i=0; i<m; i++){ for(int t=0; t<n; t++){ matZboot(t,i)=matZ(vecInd_boot(t),i); }}
    rho0_boot = corr_mat(matZboot);
    for (int j = 0.15*n; j < 0.85*n+1; j++) {
      MatrixXd matZsub1(j,m);
      MatrixXd matZsub2(n-j,m);
      for(int i=0; i<m; i++){
        for(int t=0; t<j; t++){
          matZsub1(t,i)=matZ(vecInd_boot(t),i);}
        for(int t=j; t<n; t++){
          matZsub2(t-j,i)=matZ(vecInd_boot(t),i);}
      }
      rho1_boot = corr_mat(matZsub1);
      rho2_boot = corr_mat(matZsub2);
      vecLR[j]= n*log(rho0_boot.determinant()) - j*log(rho1_boot.determinant()) - (n-j)*log(rho2_boot.determinant());
    }
    vecSUPLRboot[b] = max(vecLR);
  }
  double pval_boot = mean(vecSUPLRboot > suplr);
  // return list with test result
  return List::create(
    Named("pval_boot")=pval_boot,
    Named("suplr")=suplr,
    Named("lambda")=lambda+1,
    Named("rho0")=wrap(rho0),
    Named("rho1")=wrap(rho1),
    Named("rho2")=wrap(rho2));
}

// two-step suplr-test for constant correlation matrix against single break
// matX is tested for constant mean/variance in each dimension against single break
List suplr_GaussMarginal_GaussJoint_asym(MatrixXd matX){
  const int n = matX.rows();
  const int m = matX.cols();
  MatrixXd matZ(n,m);
  int lambda_marg[m];
  double suplr_marg[m];
  for(int i = 0; i<m; i++){
    VectorXd vecX = matX.col(i);
    List param = suplr_GaussMarginal(vecX);
    suplr_marg[i] = param["suplr"];
    lambda_marg[i] = param["lambda"];
    if(suplr_marg[i] > 11.72){
      double mu1 = param["mu1"];
      double mu2 = param["mu2"];
      double sigma1 = pow(param["sigma1"],0.5);
      double sigma2 = pow(param["sigma2"],0.5);
      for(int t=0; t<lambda_marg[i]; t++){ matZ(t,i)=(matX(t,i)-mu1)/sigma1; }
      for(int t=lambda_marg[i]; t<n; t++){ matZ(t,i)=(matX(t,i)-mu2)/sigma2; }
    }
    else {
      double mu0 = param["mu0"];
      double sigma0 = pow(param["sigma0"],0.5);
      for(int t=0; t<n; t++){ matZ(t,i)=(matX(t,i)-mu0)/sigma0; }
    }
  }
  MatrixXd rho0 = corr_mat(matZ);
  MatrixXd rho1(m,m);
  MatrixXd rho2(m,m);
  // initialize target vector
  NumericVector vecLR(n);
  // calculate lr-statistic for set of change-point candidates
  for (int j = 0.15*n; j < 0.85*n+1; j++) {
  MatrixXd matZsub1(j,m);
  MatrixXd matZsub2(n-j,m);
  for(int i=0; i<m; i++){
    for(int t=0; t<j; t++){
      matZsub1(t,i)=matZ(t,i);}
    for(int t=j; t<n; t++){
      matZsub2(t-j,i)=matZ(t,i);}
    }
  rho1 = corr_mat(matZsub1);
  rho2 = corr_mat(matZsub2);
  vecLR[j]= n*log(rho0.determinant()) - j*log(rho1.determinant()) - (n-j)*log(rho2.determinant());
  }
  double suplr = max(vecLR);
  int lambda = which_max(vecLR)+1;
  // partial sample ML-estimator
  MatrixXd matZ1(lambda,m);
  MatrixXd matZ2(n-lambda,m);
  for(int i=0; i<m; i++){
    for(int t=0; t<lambda; t++){
      matZ1(t,i)=matZ(t,i);}
    for(int t=lambda; t<n; t++){
      matZ2(t-lambda,i)=matZ(t,i);}
  }
  rho1 = corr_mat(matZ1);
  rho2 = corr_mat(matZ2);
  // convert for output
  NumericVector vecSuplr_marg(m);
  IntegerVector vecLambda_marg(m);
  for(int i=0; i<m; i++){
    vecSuplr_marg[i]=suplr_marg[i];
    vecLambda_marg[i]=lambda_marg[i];
  }
  // return list with test result
  return List::create(
      Named("suplr")=suplr,
      Named("lambda")=lambda+1,
      Named("matZ")=matZ,
      Named("suplr_margins")=vecSuplr_marg,
      Named("lambda_margins")=vecLambda_marg,
      Named("rho0")=wrap(rho0),
      Named("rho1")=wrap(rho1),
      Named("rho2")=wrap(rho2));
}


// two-step suplr-test for constant correlation under Gaussian Distribution against single break using residual bootstrap
// matX is tested for constant mean/variance in each dimension against single break
List suplr_GaussMarginal_GaussJoint_resboot(MatrixXd matX, const int B = 200){
  const int n = matX.rows();
  const int m = matX.cols();
  IntegerVector vecInd(n);
  for(int t=0; t<n; t++){vecInd[t]=t;}
  MatrixXd matZ(n,m);
  int lambda_marg[m];
  double suplr_marg[m];
  for(int i = 0; i<m; i++){
    VectorXd vecX = matX.col(i);
    List param = suplr_GaussMarginal(vecX);
    suplr_marg[i] = param["suplr"];
    lambda_marg[i] = param["lambda"];
    if(suplr_marg[i] > 11.72){
      double mu1 = param["mu1"];
      double mu2 = param["mu2"];
      double sigma1 = pow(param["sigma1"],0.5);
      double sigma2 = pow(param["sigma2"],0.5);
      for(int t=0; t<lambda_marg[i]; t++){ matZ(t,i)=(matX(t,i)-mu1)/sigma1; }
      for(int t=lambda_marg[i]; t<n; t++){ matZ(t,i)=(matX(t,i)-mu2)/sigma2; }
    }
    else{
      double mu0 = param["mu0"];
      double sigma0 = pow(param["sigma0"],0.5);
      for(int t=0; t<n; t++){ matZ(t,i)=(matX(t,i)-mu0)/sigma0; }
    }
  }
  MatrixXd rho0 = corr_mat(matZ);
  MatrixXd rho1(m,m);
  MatrixXd rho2(m,m);
  // initialize target vector
  NumericVector vecLR(n);
  // calculate lr-statistic for set of change-point candidates
  for (int j = 0.15*n; j < 0.85*n+1; j++) {
    MatrixXd matZsub1(j,m);
    MatrixXd matZsub2(n-j,m);
    for(int i=0; i<m; i++){
      for(int t=0; t<j; t++){
        matZsub1(t,i)=matZ(t,i);}
      for(int t=j; t<n; t++){
        matZsub2(t-j,i)=matZ(t,i);}
    }
    rho1 = corr_mat(matZsub1);
    rho2 = corr_mat(matZsub2);
    vecLR[j]= n*log(rho0.determinant()) - j*log(rho1.determinant()) - (n-j)*log(rho2.determinant());
  }
  double suplr = max(vecLR);
  int lambda = which_max(vecLR)+1;
  // residual bootstrap
  NumericVector vecSUPLRboot(B);
  MatrixXd rho0_boot;
  MatrixXd rho1_boot;
  MatrixXd rho2_boot;
  MatrixXd matZboot(n,m);
  for(int b=0; b<B; b++){
    IntegerVector vecInd_boot = boot(vecInd,n);
    for(int i=0; i<m; i++){ for(int t=0; t<n; t++){ matZboot(t,i)=matZ(vecInd_boot(t),i); }}
    rho0_boot = corr_mat(matZboot);
    for (int j = 0.15*n; j < 0.85*n+1; j++) {
      MatrixXd matZsub1(j,m);
      MatrixXd matZsub2(n-j,m);
      for(int i=0; i<m; i++){
        for(int t=0; t<j; t++){
          matZsub1(t,i)=matZ(vecInd_boot(t),i);}
        for(int t=j; t<n; t++){
          matZsub2(t-j,i)=matZ(vecInd_boot(t),i);}
      }
      rho1_boot = corr_mat(matZsub1);
      rho2_boot = corr_mat(matZsub2);
      vecLR[j]= n*log(rho0_boot.determinant()) - j*log(rho1_boot.determinant()) - (n-j)*log(rho2_boot.determinant());
    }
    vecSUPLRboot[b] = max(vecLR);
  }
  double pval_boot = mean(vecSUPLRboot > suplr);
  // convert for output
  NumericVector vecSuplr_marg(m);
  IntegerVector vecLambda_marg(m);
  for(int i=0; i<m; i++){
    vecSuplr_marg[i]=suplr_marg[i];
    vecLambda_marg[i]=lambda_marg[i];}
  // return list with test result
  return List::create(
    Named("pval_boot") = pval_boot,
    Named("suplr") = suplr,
    Named("lambda") = lambda,
    Named("matZ")=matZ,
    Named("suplr_margins")=vecSuplr_marg,
    Named("lambda_margins")=vecLambda_marg,
    Named("rho0") = rho0,
    Named("rho1") = rho1,
    Named("rho2") = rho2);
}

#endif
