#ifndef __SUPLR_T_MARGIN_INCLUDED__
#define __SUPLR_T_MARGIN_INCLUDED__
#include <Rcpp.h>
#include <RcppEigen.h>
#include <cmath>
#include "0_SeqStrucChange_aux.h"
#include "t_aux.h"

// [[Rcpp::depends(RcppEigen)]]
using namespace Rcpp;
using namespace Eigen;

using Eigen::Map;
using Eigen::MatrixXd;
using Eigen::VectorXd;

// lr-test for constant univeriate t-distribution against single break
List suplr_tMarginal(VectorXd vecX, double nu=0, bool nu_const = true, int niter=100){
  const int n = vecX.size();
  // moments under the null
  List param = mle_t(vecX,nu,niter);
  double mu0 = param["Mu"];
  double xi0 = param["Xi"];
  double nu0 = param["nu"];
  double ll0 = logl_t(vecX,mu0,xi0,nu0);
  if(nu_const == true) nu=nu0;
  // initialize target array
  NumericVector vecLR(n);
  double ll1;
  double ll2;
  List param1;
  List param2;
  // loop over potential breakpoints
  for (int j = 0.15*n; j < 0.85*n+1; j++) {
    VectorXd vecXsub1(j);
    for(int t=0; t<j; t++){vecXsub1(t)=vecX(t);}
    VectorXd vecXsub2(n-j);
    for(int t=j; t<n; t++){vecXsub2(t-j)=vecX(t);}
    param1 = mle_t(vecXsub1,nu,1);
    param2 = mle_t(vecXsub2,nu,1);
    // evaluate log-likelihood udner alternative
    ll1=logl_t(vecXsub1,param1["Mu"],param1["Xi"],param1["nu"]);
    ll2=logl_t(vecXsub2,param2["Mu"],param2["Xi"],param2["nu"]);
    if(2*(ll1+ll2-ll0) > 0){vecLR[j] = 2*(ll1+ll2-ll0);}
    else{vecLR[j] = 0;}
  }
  double suplr = max(vecLR);
  int lambda = which_max(vecLR)+1;
  // partial sample ML-estimator (part 1)
  VectorXd vecXsub1(lambda);
  for(int t=0; t<lambda; t++){vecXsub1(t)=vecX(t);}
  param1=mle_t(vecXsub1,nu,niter);
  double mu1 = param1["Mu"];
  double xi1 = param1["Xi"];
  double nu1 = param1["nu"];
  // partial sample ML-estimator (part 2)
  VectorXd vecXsub2(n-lambda);
  for(int t=lambda; t<n; t++){vecXsub2(t-lambda)=vecX(t);}
  param2=mle_t(vecXsub2,nu,niter);
  double mu2 = param2["Mu"];
  double xi2 = param2["Xi"];
  double nu2 = param2["nu"];
  // return list with test result
  return List::create(
    Named("suplr")=suplr,
    Named("lambda")=lambda,
    Named("nu0")=nu0,
    Named("nu1")=nu1,
    Named("nu2")=nu2,
    Named("mu0")=mu0,
    Named("mu1")=mu1,
    Named("mu2")=mu2,
    Named("xi0")=xi0,
    Named("xi1")=xi1,
    Named("xi2")=xi2);
}


#endif
